"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chatbox = void 0;
const react_1 = require("@chakra-ui/react");
const io_1 = require("react-icons/io");
const MyFlex_1 = require("./MyFlex");
const chat_1 = require("@strata-foundation/chat");
const react_2 = require("@strata-foundation/react");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const react_3 = __importStar(require("react"));
const ai_1 = require("react-icons/ai");
const hooks_1 = require("../hooks");
const reply_1 = require("../contexts/reply");
const BuyMoreButton_1 = require("./BuyMoreButton");
const ChatInput_1 = require("./ChatInput");
const GifSearch_1 = require("./GifSearch");
const showdown_1 = require("showdown");
const react_hot_toast_1 = __importDefault(require("react-hot-toast"));
const LongPromiseNotification_1 = require("./LongPromiseNotification");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const wallet_adapter_react_ui_1 = require("@solana/wallet-adapter-react-ui");
const LoadWalletModal_1 = require("./LoadWalletModal");
const useChatOwnedAmount_1 = require("../hooks/useChatOwnedAmount");
const useEmojiSearch_1 = require("../hooks/useEmojiSearch");
const Files_1 = require("./Files");
const ReplyBar_1 = require("./ReplyBar");
const react_async_hook_1 = require("react-async-hook");
const converter = new showdown_1.Converter({
    simpleLineBreaks: true,
});
const popoverWidth = {
    base: "full",
    md: "50%",
};
const DARK_BG = {
    bg: "linear-gradient(0deg, rgba(17,24,39) 40%, rgba(21,24,38,0) 100%)",
};
function Chatbox({ scrollRef, chatKey, onAddPendingMessage, files, setFiles, onUploadFile, }) {
    const inputRef = (0, react_3.useRef)(null);
    const [input, setInput] = (0, react_3.useState)("");
    const { emojis, search, searchMatch, reset: resetEmoji } = (0, useEmojiSearch_1.useEmojiSearch)();
    const { isOpen, onToggle, onClose } = (0, react_1.useDisclosure)();
    const { isOpen: loadWalletIsOpen, onOpen: onOpenLoadWallet, onClose: onCloseLoadWallet, } = (0, react_1.useDisclosure)();
    const { connected, publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const { setVisible } = (0, wallet_adapter_react_ui_1.useWalletModal)();
    const { account: profileAccount } = (0, hooks_1.useWalletProfile)(publicKey || undefined);
    const { delegateWallet, needsTopOff, needsInit, loadDelegate, loading: loadingDelegate, error: delegateError, } = (0, hooks_1.useLoadDelegate)();
    const { isOpen: delegateIsOpen, onClose: closeDelegate, onOpen: openDelegate, } = (0, react_1.useDisclosure)({
        defaultIsOpen: false,
    });
    const gaEventTracker = (0, hooks_1.useAnalyticsEventTracker)();
    const chatBg = (0, react_1.useColorModeValue)("gray.100", "gray.800");
    const { handleErrors } = (0, react_2.useErrorHandler)();
    const { info: chat } = (0, hooks_1.useChat)(chatKey);
    const { metadata: readMetadata, image: readImage } = (0, react_2.useTokenMetadata)(chat === null || chat === void 0 ? void 0 : chat.readPermissionKey);
    const { metadata: postMetadata, image: postImage } = (0, react_2.useTokenMetadata)(chat === null || chat === void 0 ? void 0 : chat.postPermissionKey);
    const { amount: ownedAmount } = (0, useChatOwnedAmount_1.useChatOwnedAmount)(publicKey || undefined, chatKey);
    const mint = (0, react_2.useMint)(chat === null || chat === void 0 ? void 0 : chat.postPermissionKey);
    const postAmount = (chat === null || chat === void 0 ? void 0 : chat.postPermissionAmount) &&
        mint &&
        (0, spl_token_bonding_1.toNumber)(chat === null || chat === void 0 ? void 0 : chat.postPermissionAmount, mint);
    const hasEnough = typeof postAmount == "undefined" ||
        typeof ownedAmount == "undefined" ||
        ownedAmount >= postAmount;
    const [loading, setLoading] = (0, react_3.useState)(false);
    const { sendMessage: sendMessageImpl, error } = (0, hooks_1.useSendMessage)({
        chatKey,
        onAddPendingMessage: (msg) => {
            setLoading(false);
            scrollRef.current.scrollTop = 0;
            if (onAddPendingMessage) {
                onAddPendingMessage(msg);
            }
        },
    });
    const onCancelFile = (0, react_3.useCallback)((file) => setFiles((files) => files.filter((f) => f.file != file)), [setFiles]);
    const { replyMessage, hideReply } = (0, reply_1.useReply)();
    (0, react_3.useEffect)(() => {
        var _a;
        if (replyMessage)
            (_a = inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, [replyMessage]);
    const { execute: sendMessage } = (0, react_async_hook_1.useAsyncCallback)((m) => __awaiter(this, void 0, void 0, function* () {
        setInput("");
        resetEmoji();
        setLoading(true);
        hideReply();
        try {
            if (replyMessage === null || replyMessage === void 0 ? void 0 : replyMessage.id)
                m.referenceMessageId = replyMessage === null || replyMessage === void 0 ? void 0 : replyMessage.id;
            // Show toast if uploading files
            if (m.fileAttachments && m.fileAttachments.length > 0) {
                const text = `Uploading ${m.fileAttachments.map((f) => f.name)} to SHDW Drive...`;
                react_hot_toast_1.default.custom((t) => (react_3.default.createElement(LongPromiseNotification_1.LongPromiseNotification, { estTimeMillis: 1 * 60 * 1000, text: text, onError: (e) => {
                        handleErrors(e);
                        react_hot_toast_1.default.dismiss(t.id);
                    }, exec: () => __awaiter(this, void 0, void 0, function* () {
                        yield sendMessageImpl(m);
                        return true;
                    }), onComplete: () => __awaiter(this, void 0, void 0, function* () {
                        react_hot_toast_1.default.dismiss(t.id);
                    }) })), {
                    duration: Infinity,
                });
                setFiles([]);
            }
            else {
                yield sendMessageImpl(m);
            }
        }
        finally {
            setLoading(false);
        }
        gaEventTracker({
            action: "Send Message",
        });
    }));
    const handleChange = (0, react_3.useCallback)((e) => __awaiter(this, void 0, void 0, function* () {
        const content = e.currentTarget.value;
        search(e);
        setInput(content);
    }), [setInput, search]);
    const handleKeyDown = (0, react_3.useCallback)((ev) => {
        if (ev.key === "Enter") {
            if (!ev.shiftKey) {
                ev.preventDefault();
                sendMessage({
                    type: chat_1.MessageType.Html,
                    html: converter.makeHtml(input.replace("\n", "\n\n")),
                    fileAttachments: files,
                });
            }
        }
    }, [sendMessage, files, input]);
    const handleSendClick = (0, react_3.useCallback)(() => sendMessage({
        type: chat_1.MessageType.Html,
        html: converter.makeHtml(input),
        fileAttachments: files,
    }), [sendMessage, input, files]);
    const handleEmojiClick = (native) => {
        var _a;
        setInput([`:${searchMatch}`, `:${searchMatch}:`].reduce((acc, string) => acc.replace(string, native), input));
        (_a = inputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
        resetEmoji();
    };
    handleErrors(error, delegateError);
    return (react_3.default.createElement(MyFlex_1.Flex, { w: "full", position: "relative" }, !connected || !hasEnough || needsTopOff ? (react_3.default.createElement(MyFlex_1.Flex, { position: "absolute", bottom: "0", pb: 12, pt: 40, w: "full", justify: "center", bg: "linear-gradient(0deg, rgba(255,255,255) 40%, rgba(255,255,255,0) 100%)", _dark: DARK_BG },
        react_3.default.createElement(react_1.VStack, { w: "full", h: "full", justify: "center", align: "center", maxW: "360px" }, !connected ? (react_3.default.createElement(react_3.default.Fragment, null,
            react_3.default.createElement(react_1.Button, { size: "md", colorScheme: "primary", onClick: () => setVisible(true), px: 16 }, "Connect Wallet"))) : !hasEnough ? (react_3.default.createElement(react_3.default.Fragment, null,
            react_3.default.createElement(react_1.Text, { fontWeight: "bold", align: "center" }, "In order to participate in this chat:"),
            react_3.default.createElement(react_1.Box, { w: "full", fontSize: "sm" },
                readMetadata && (react_3.default.createElement(react_1.HStack, { spacing: 1 },
                    react_3.default.createElement(react_1.Text, null, "Read Message"),
                    react_3.default.createElement(MyFlex_1.Flex, { grow: 1 },
                        react_3.default.createElement(react_1.Divider, { variant: "dashed" })),
                    react_3.default.createElement(react_1.Text, { fontWeight: "bold", textTransform: "capitalize" }, "Hold 1"),
                    react_3.default.createElement(react_1.Avatar, { w: "18px", h: "18px", title: readMetadata === null || readMetadata === void 0 ? void 0 : readMetadata.data.symbol, src: readImage }))),
                postMetadata && (react_3.default.createElement(react_1.HStack, { spacing: 1 },
                    react_3.default.createElement(react_1.Text, null, "Post Message"),
                    react_3.default.createElement(MyFlex_1.Flex, { grow: 1 },
                        react_3.default.createElement(react_1.Divider, { variant: "dashed" })),
                    react_3.default.createElement(react_1.Text, { fontWeight: "bold", textTransform: "capitalize" },
                        Object.keys((chat === null || chat === void 0 ? void 0 : chat.postPermissionAction) || {})[0],
                        " ",
                        postAmount),
                    react_3.default.createElement(react_1.Avatar, { w: "18px", h: "18px", title: postMetadata === null || postMetadata === void 0 ? void 0 : postMetadata.data.symbol, src: postImage })))),
            react_3.default.createElement(react_1.Box, { w: "full", fontSize: "sm" }, readMetadata && (react_3.default.createElement(react_1.HStack, { spacing: 1 },
                react_3.default.createElement(react_1.Text, null, "You currently have"),
                react_3.default.createElement(MyFlex_1.Flex, { grow: 1 },
                    react_3.default.createElement(react_1.Divider, { variant: "dashed" })),
                react_3.default.createElement(react_1.Text, { fontWeight: "bold", textTransform: "capitalize" }, ownedAmount ? (0, react_2.roundToDecimals)(ownedAmount, 4) : 0),
                react_3.default.createElement(react_1.Avatar, { w: "18px", h: "18px", title: readMetadata === null || readMetadata === void 0 ? void 0 : readMetadata.data.symbol, src: readImage })))),
            react_3.default.createElement(react_1.Box, { pt: 4 },
                react_3.default.createElement(BuyMoreButton_1.BuyMoreButton, { mint: chat === null || chat === void 0 ? void 0 : chat.postPermissionKey, btnProps: { px: 16, size: "md", variant: "solid" } })))) : needsTopOff ? (react_3.default.createElement(react_3.default.Fragment, null,
            react_3.default.createElement(LoadWalletModal_1.LoadWalletModal, { isOpen: loadWalletIsOpen, onClose: onCloseLoadWallet, onLoaded: () => onCloseLoadWallet() }),
            react_3.default.createElement(MyFlex_1.Flex, { justify: "center", mb: "6px" },
                react_3.default.createElement(react_1.Button, { isLoading: loadingDelegate, size: "md", colorScheme: "primary", onClick: () => onOpenLoadWallet(), px: 16 }, "Top Off Chat Wallet")))) : null))) : (react_3.default.createElement(react_1.VStack, { w: "full" },
        !delegateWallet && (react_3.default.createElement(react_1.HStack, { mb: -3, mt: 1, fontSize: "sm" },
            react_3.default.createElement(react_1.Text, { fontWeight: "bold" }, "Tired of approving transactions?"),
            react_3.default.createElement(react_1.Button, { fontSize: "sm", variant: "link", size: "md", colorScheme: "primary", onClick: () => openDelegate(), px: 16 }, "Load Delegate Wallet"))),
        (needsTopOff || needsInit) && (react_3.default.createElement(LoadWalletModal_1.LoadWalletModal, { isOpen: delegateIsOpen, onClose: closeDelegate, onLoaded: closeDelegate })),
        react_3.default.createElement(MyFlex_1.Flex, { direction: "column", position: "sticky", bottom: 0, p: 2, w: "full", minH: "76px" },
            react_3.default.createElement(react_1.Popover, { matchWidth: true, isOpen: emojis.length > 0, placement: "top", autoFocus: false, closeOnBlur: false },
                react_3.default.createElement(react_1.PopoverTrigger, null,
                    react_3.default.createElement(MyFlex_1.Flex, { w: "full" })),
                react_3.default.createElement(react_1.PopoverContent, { bg: chatBg, border: "none", w: popoverWidth },
                    react_3.default.createElement(react_1.PopoverBody, { px: 0, pt: 0 },
                        react_3.default.createElement(react_1.VStack, { spacing: 0, w: "full", align: "start" },
                            react_3.default.createElement(react_1.Text, { p: 2, fontSize: "xs", fontWeight: "bold", textTransform: "uppercase", lineHeight: "normal" },
                                "Emojis Matching :",
                                react_3.default.createElement(react_1.Text, { as: "span", textTransform: "none" }, searchMatch)),
                            react_3.default.createElement(react_1.Divider, null),
                            emojis.map((e, indx) => (react_3.default.createElement(react_1.HStack, { w: "full", p: 2, key: e.name, onClick: () => handleEmojiClick(e.skins[0].native), _hover: {
                                    cursor: "pointer",
                                    bg: "gray.200",
                                    _dark: {
                                        bg: "gray.700",
                                    },
                                } },
                                react_3.default.createElement(react_1.Text, { fontSize: "xl" }, e.skins[0].native),
                                react_3.default.createElement(react_1.Text, { fontSize: "sm" }, e.name)))))))),
            react_3.default.createElement(react_1.VStack, { p: "10px", spacing: 2, w: "full", align: "left", bg: chatBg, rounded: "lg" },
                react_3.default.createElement(Files_1.Files, { files: files, onCancelFile: onCancelFile }),
                react_3.default.createElement(ReplyBar_1.ReplyBar, null),
                react_3.default.createElement(react_1.HStack, { w: "full" },
                    react_3.default.createElement(ChatInput_1.ChatInput, { inputRef: inputRef, onChange: handleChange, value: input, onKeyDown: handleKeyDown }),
                    react_3.default.createElement(react_1.IconButton, { isLoading: loading, "aria-label": "Select Image", variant: "outline", onClick: onUploadFile, icon: react_3.default.createElement(react_1.Icon, { w: "24px", h: "24px", as: io_1.IoMdAttach }) }),
                    react_3.default.createElement(react_1.IconButton, { "aria-label": "Select GIF", variant: "outline", onClick: onToggle, icon: react_3.default.createElement(react_1.Icon, { w: "24px", h: "24px", as: ai_1.AiOutlineGif }) }),
                    react_3.default.createElement(react_1.Button, { isLoading: loading, colorScheme: "primary", variant: "outline", isDisabled: !hasEnough || (!input && files.length == 0), onClick: handleSendClick },
                        react_3.default.createElement(react_1.Icon, { as: ai_1.AiOutlineSend }))))),
        react_3.default.createElement(react_1.Modal, { isOpen: isOpen, onClose: onClose, size: "2xl", isCentered: true, trapFocus: true },
            react_3.default.createElement(react_1.ModalOverlay, null),
            react_3.default.createElement(react_1.ModalContent, { borderRadius: "xl", shadow: "xl" },
                react_3.default.createElement(react_1.ModalHeader, null, "Select GIF"),
                react_3.default.createElement(react_1.ModalBody, null,
                    react_3.default.createElement(GifSearch_1.GifSearch, { onSelect: (gifyId) => {
                            onClose();
                            sendMessage({ type: chat_1.MessageType.Gify, gifyId });
                        } }))))))));
}
exports.Chatbox = Chatbox;
//# sourceMappingURL=Chatbox.js.map