"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BasicInfo = void 0;
const react_1 = __importStar(require("react"));
const react_hook_form_1 = require("react-hook-form");
const yup_1 = require("@hookform/resolvers/yup");
const yup = __importStar(require("yup"));
const react_2 = require("@chakra-ui/react");
const ri_1 = require("react-icons/ri");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const chat_1 = require("@strata-foundation/chat");
const react_3 = require("@strata-foundation/react");
const react_hot_toast_1 = __importDefault(require("react-hot-toast"));
const contexts_1 = require("../../contexts");
const hooks_1 = require("../../hooks");
const LongPromiseNotification_1 = require("../LongPromiseNotification");
const form_1 = require("../form");
const validationSchema = yup
    .object({
    name: yup.string().required().max(28),
    identifier: yup
        .string()
        .required()
        .min(6)
        .max(28)
        .matches(/^[a-zA-Z0-9\_]+$/g, "must be alphanumeric and not have any spaces"),
    image: yup.mixed(),
    imageUrl: yup.string(),
})
    .required();
const BasicInfo = ({ state, setState, onBack, onNext, }) => {
    var _a, _b, _c, _d;
    const { publicKey: connectedWallet } = (0, wallet_adapter_react_1.useWallet)();
    const { chatSdk } = (0, contexts_1.useChatSdk)();
    const { handleErrors } = (0, react_3.useErrorHandler)();
    const { result: chatStorage } = (0, hooks_1.useChatStorageAccountKey)();
    const hiddenFileInput = (0, react_1.useRef)(null);
    const [imgUrl, setImgUrl] = (0, react_1.useState)();
    const [isValidIdentifier, setIsValidIdentifier] = (0, react_1.useState)(null);
    const { delegateWallet, error: delegateError } = (0, hooks_1.useLoadDelegate)();
    handleErrors(delegateError);
    const { register, handleSubmit, watch, formState: { errors }, setValue, setError, clearErrors, } = (0, react_hook_form_1.useForm)({
        mode: "onChange",
        resolver: (0, yup_1.yupResolver)(validationSchema),
        defaultValues: Object.assign({}, state.wizardData),
    });
    const { name, identifier, image, imageUrl } = watch();
    const { wallet: identifierOwner } = (0, hooks_1.useWalletFromChatIdentifier)(identifier);
    const inputBg = { bg: "gray.200", _dark: { bg: "gray.800" } };
    const helpTextColor = { color: "black", _dark: { color: "gray.400" } };
    const handleImageChange = (e) => {
        const file = e.target.files[0];
        setValue("image", file || null);
        clearErrors("image");
    };
    const verifyIdentifier = (0, react_1.useCallback)((identifier, identifierOwner, connectedWallet) => {
        if (identifier === "") {
            setIsValidIdentifier(null);
            clearErrors("identifier");
        }
        else {
            if (identifier.length >= 6) {
                const ownsIdentifier = identifierOwner === null || identifierOwner === void 0 ? void 0 : identifierOwner.equals(connectedWallet);
                const isValid = !identifierOwner || ownsIdentifier;
                if (isValid) {
                    setIsValidIdentifier(true);
                    if (ownsIdentifier) {
                        setState({
                            wizardData: Object.assign(Object.assign({}, state.wizardData), { ownsIdentifier: true }),
                        });
                    }
                }
                else {
                    setIsValidIdentifier(false);
                    setError("identifier", {
                        message: "Chat identifier is already taken!",
                    });
                }
            }
        }
    }, []);
    (0, react_1.useEffect)(() => {
        verifyIdentifier(identifier, identifierOwner, connectedWallet);
    }, [identifier, identifierOwner]);
    const onSubmit = (data) => {
        setState({
            wizardData: Object.assign(Object.assign({}, state.wizardData), data),
        });
        onNext();
    };
    (0, react_1.useEffect)(() => {
        if (image) {
            const reader = new FileReader();
            reader.onload = (event) => {
                var _a;
                setImgUrl(((_a = event.target) === null || _a === void 0 ? void 0 : _a.result) || "");
            };
            reader.readAsDataURL(image);
            if (!imageUrl) {
                const text = `Uploading ${image.name} to SHDW Drive...`;
                (0, chat_1.randomizeFileName)(image);
                const url = `https://shdw-drive.genesysgo.net/${chatStorage}/${image.name}`;
                setValue("imageUrl", url);
                react_hot_toast_1.default.custom((t) => (react_1.default.createElement(LongPromiseNotification_1.LongPromiseNotification, { estTimeMillis: 2 * 60 * 1000, text: text, onError: (e) => {
                        handleErrors(e);
                        react_hot_toast_1.default.dismiss(t.id);
                    }, exec: () => __awaiter(void 0, void 0, void 0, function* () {
                        yield (0, chat_1.uploadFiles)(chatSdk.provider, [image], delegateWallet);
                        return true;
                    }), onComplete: () => __awaiter(void 0, void 0, void 0, function* () {
                        const images = document.querySelectorAll(`img[src*="${url}"]`);
                        images.forEach((image) => {
                            // @ts-ignore
                            image.src = url;
                        });
                        react_hot_toast_1.default.dismiss(t.id);
                    }) })), {
                    duration: Infinity,
                });
            }
        }
        else {
            setImgUrl(undefined);
        }
    }, [image]);
    return (react_1.default.createElement("form", { onSubmit: handleSubmit(onSubmit) },
        react_1.default.createElement(react_2.VStack, { w: "full", alignItems: "start", gap: 6, spacing: 0 },
            react_1.default.createElement(react_2.Box, null,
                react_1.default.createElement(react_2.Text, { fontWeight: "bold", fontSize: "md" }, "Let's Start with the Basic info"),
                react_1.default.createElement(react_2.Text, { fontSize: "xs", fontWeight: "normal" }, "What do you want your chat to be called?")),
            react_1.default.createElement(form_1.FormControlWithError, { id: "name", label: "Name", errors: errors, help: "The name that will apper in the sidebar." },
                react_1.default.createElement(react_2.Input, Object.assign({ id: "name", variant: "filled" }, inputBg, register("name")))),
            react_1.default.createElement(react_2.FormControl, { isInvalid: !!((_a = errors.identifier) === null || _a === void 0 ? void 0 : _a.message) },
                react_1.default.createElement(react_2.FormLabel, { htmlFor: "identifier" }, "Identifier"),
                react_1.default.createElement(react_2.InputGroup, null,
                    react_1.default.createElement(react_2.Input, Object.assign({ id: "identifier", variant: "filled" }, inputBg, register("identifier"))),
                    isValidIdentifier && (react_1.default.createElement(react_2.InputRightElement, null,
                        react_1.default.createElement(ri_1.RiCheckLine, { fontSize: "1.5rem" })))),
                !((_b = errors.identifier) === null || _b === void 0 ? void 0 : _b.message) ? (react_1.default.createElement(react_2.FormHelperText, Object.assign({ fontSize: "xs" }, helpTextColor, { alignItems: "center", justifyContent: "center" }), isValidIdentifier ? (react_1.default.createElement(react_2.Flex, { alignItems: "center", color: "green.500" },
                    react_1.default.createElement(react_2.Icon, { as: ri_1.RiCheckboxCircleFill, mr: 1, fontSize: "1.3rem" }),
                    react_1.default.createElement(react_2.Box, null, "Chat identifier is available!"))) : ("A NFT representing ownership of the chat."))) : (react_1.default.createElement(react_2.FormErrorMessage, { fontSize: "xs", textTransform: "capitalize" },
                    react_1.default.createElement(react_2.Icon, { as: ri_1.RiErrorWarningFill, mr: 1, fontSize: "1.3rem" }),
                    errors.identifier.message))),
            react_1.default.createElement(react_2.FormControl, { isInvalid: !!((_c = errors.image) === null || _c === void 0 ? void 0 : _c.message) },
                react_1.default.createElement(react_2.FormLabel, null, "Upload Picture"),
                react_1.default.createElement(react_2.HStack, { w: "full", spacing: 4 },
                    react_1.default.createElement(react_2.Button, { size: "sm", colorScheme: "primary", variant: "outline", onClick: () => hiddenFileInput.current.click() }, "Choose Image"),
                    image && (react_1.default.createElement(react_2.HStack, { spacing: 2, align: "center" },
                        react_1.default.createElement(react_2.Image, { alt: image === null || image === void 0 ? void 0 : image.name, w: "32px", h: "32px", src: imgUrl }),
                        react_1.default.createElement(react_2.Text, { color: "gray.500" }, image === null || image === void 0 ? void 0 : image.name),
                        react_1.default.createElement(react_2.Icon, { w: "22px", h: "22px", color: "green.400", as: ri_1.RiCheckboxCircleFill })))),
                react_1.default.createElement("input", { id: "image", type: "file", accept: ".png,.jpg,.gif,.mp4,.svg", multiple: false, onChange: handleImageChange, ref: hiddenFileInput, style: { display: "none" } }),
                !((_d = errors.image) === null || _d === void 0 ? void 0 : _d.message) ? (react_1.default.createElement(react_2.FormHelperText, Object.assign({ fontSize: "xs" }, helpTextColor), "The image that will appear in the sidebar.")) : (react_1.default.createElement(react_2.FormErrorMessage, { fontSize: "xs", textTransform: "capitalize" },
                    react_1.default.createElement(react_2.Icon, { as: ri_1.RiErrorWarningFill, mr: 2, fontSize: "1.3rem" }),
                    errors.image.message))),
            react_1.default.createElement(react_2.Flex, { align: "center", w: "full" },
                react_1.default.createElement(react_2.Divider, { borderColor: "gray.500" }),
                react_1.default.createElement(react_2.Text, { px: 4 }, "OR"),
                react_1.default.createElement(react_2.Divider, { borderColor: "gray.500" })),
            react_1.default.createElement(form_1.FormControlWithError, { id: "imageUrl", help: "A url to the image to use for your chat", label: "Image URL", errors: errors },
                react_1.default.createElement(react_2.Input, Object.assign({ variant: "filled" }, inputBg, register("imageUrl")))),
            react_1.default.createElement(react_2.ButtonGroup, { variant: "outline", colorScheme: "primary", w: "full" },
                react_1.default.createElement(react_2.Button, { w: "full", onClick: onBack }, "Back"),
                react_1.default.createElement(react_2.Button, { w: "full", variant: "solid", type: "submit", disabled: !name || !isValidIdentifier || (!imageUrl && !image) }, "Next")))));
};
exports.BasicInfo = BasicInfo;
//# sourceMappingURL=BasicInfo.js.map