"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Summary = void 0;
const react_1 = __importDefault(require("react"));
const react_hook_form_1 = require("react-hook-form");
const react_2 = require("@chakra-ui/react");
const CreateChatModal_1 = require("./CreateChatModal");
const LabelCodeValue = ({ label, value, children, }) => (react_1.default.createElement(react_2.Flex, { gap: "2" },
    react_1.default.createElement(react_2.Code, { alignItems: "center", fontWeight: "bold", textTransform: "capitalize", lineHeight: "normal", py: 1, px: 2, display: "flex", flexShrink: 0 },
        label,
        ":"),
    value && (react_1.default.createElement(react_2.Tooltip, { label: value },
        react_1.default.createElement(react_2.Text, { fontSize: "lg", isTruncated: true, w: "auto" }, value))),
    children));
const NFTSummary = ({ permissionType, amount, collectionKey }) => (react_1.default.createElement(react_2.Stack, null,
    react_1.default.createElement(react_2.Box, null,
        react_1.default.createElement(react_2.Text, { fontWeight: "bold", fontSize: "md", textTransform: "capitalize" },
            permissionType,
            " permission"),
        react_1.default.createElement(react_2.Text, { fontSize: "xs", fontWeight: "normal" }, "You've decided to use an NFT collection.")),
    react_1.default.createElement(LabelCodeValue, { label: "Hold Amount", value: amount }),
    react_1.default.createElement(LabelCodeValue, { label: "Collection Key", value: collectionKey })));
const TokenSummary = ({ permissionType, isExisting, amount, mint, startingPrice }) => (react_1.default.createElement(react_2.Stack, null,
    react_1.default.createElement(react_2.Box, null,
        react_1.default.createElement(react_2.Text, { fontWeight: "bold", fontSize: "md", textTransform: "capitalize" },
            permissionType,
            " permission"),
        react_1.default.createElement(react_2.Text, { fontSize: "xs", fontWeight: "normal" }, isExisting ? (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(react_2.Text, null, "You've decided to use an existing token."))) : (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(react_2.Text, null, "You've decided to create a new token."))))),
    react_1.default.createElement(LabelCodeValue, { label: "Hold Amount", value: amount }),
    isExisting ? (react_1.default.createElement(LabelCodeValue, { label: "Existing Mint", value: mint })) : (react_1.default.createElement(LabelCodeValue, { label: "Starting Price (In SOL)", value: startingPrice }))));
const Summary = ({ state, onBack, onNext }) => {
    const { handleSubmit, formState: { errors }, } = (0, react_hook_form_1.useForm)();
    const onSubmit = (data) => {
        onNext();
    };
    const { isSubmitting, wizardData: { name, identifier, readType, readForm, postType, postForm, imageUrl, }, } = state;
    return (react_1.default.createElement("form", { onSubmit: handleSubmit(onSubmit) },
        react_1.default.createElement(react_2.Stack, { w: "full", alignItems: "start", gap: 6, spacing: 0 },
            react_1.default.createElement(react_2.Box, null,
                react_1.default.createElement(react_2.Text, { fontWeight: "bold", fontSize: "md" }, "Please verify all the information up until this point"),
                react_1.default.createElement(react_2.Text, { fontSize: "xs", fontWeight: "normal" }, "You're almost there, one step away from your own gated chat!")),
            react_1.default.createElement(react_2.Stack, { w: "full", gap: 6 },
                react_1.default.createElement(react_2.Stack, null,
                    react_1.default.createElement(react_2.Heading, { fontSize: "lg" }, "Basic Info"),
                    react_1.default.createElement(LabelCodeValue, { label: "name", value: name }),
                    react_1.default.createElement(LabelCodeValue, { label: "identifier", value: identifier }),
                    react_1.default.createElement(LabelCodeValue, { label: "image" },
                        react_1.default.createElement(react_2.Image, { alt: `${identifier}-chat-image`, w: "80px", h: "80px", src: imageUrl }))),
                react_1.default.createElement(react_2.Stack, { gap: 8 },
                    react_1.default.createElement(react_2.Stack, null, readType === CreateChatModal_1.ReadPostType.NFT ? (react_1.default.createElement(NFTSummary, Object.assign({}, readForm, { permissionType: "read" }))) : (react_1.default.createElement(TokenSummary, Object.assign({}, readForm, { permissionType: "read" })))),
                    react_1.default.createElement(react_2.Stack, null, postType === CreateChatModal_1.ReadPostType.NFT ? (react_1.default.createElement(NFTSummary, Object.assign({}, postForm, { permissionType: "post" }))) : (react_1.default.createElement(TokenSummary, Object.assign({}, postForm, { permissionType: "post" })))))),
            react_1.default.createElement(react_2.ButtonGroup, { variant: "outline", colorScheme: "primary", w: "full", isDisabled: isSubmitting },
                react_1.default.createElement(react_2.Button, { w: "full", onClick: onBack }, "Back"),
                react_1.default.createElement(react_2.Button, { w: "full", variant: "solid", type: "submit", isLoading: isSubmitting, loadingText: "Creating Chat" }, "Create Chat")))));
};
exports.Summary = Summary;
//# sourceMappingURL=Summary.js.map